clear, clc

syms alpha1 alpha2 alpha3 beta1 beta2 beta3 gamma1 gamma2 gamma3 real
syms x1 x2 real
x = [x1;x2];

% nullspace known parameters (matrix N in Equation 4) for ACs A and C
% refer to main.m for the derivation of N
a = sym('a', [3 4]);
c = sym('c', [3 4]);

Na = [a;  1  0  0   0;
    0  1  0   0;
    0  0  1   0];
Nc = [c;    1  0  0   0;
    0  1  0   0;
    0  0  1   0];

%Estimate y from A and x
A = Na*[alpha1;alpha2;alpha3;1]; A = reshape(A,[2 3]);
y = simplify(A*[x;1]);

% Estimate nullspace of AC B from essential matrix Eb and y
Eb = sym('Eb',[3 3]);

Nb = ComputeNullSpace(Eb,y);

B = Nb*[beta1;beta2;beta3;1]; B = reshape(B,[2 3]);
C = Nc*[gamma1;gamma2;gamma3;1]; C = reshape(C,[2 3]);
A = [A;0 0 1]; B = [B;0 0 1]; C = [C;0 0 1];

C_ = simplify(B*A);

%Use relation C = BA to relate unknown variables
eqs = C-C_;

eqs_all = eqs(1:2,:);
eqs_all = eqs_all(:);

% Generate system of equations 12
[G, g] = equationsToMatrix(eqs_all,[beta1 beta2 beta3 gamma1 gamma2 gamma3]);
G = simplify(G);


function N = ComputeNullSpace(E,x)
e1=E(1);e2=E(2);e3=E(3);e4=E(4);e5=E(5);e6=E(6);e7=E(7);e8=E(8);e9=E(9);
x1=x(1);x2=x(2);
N = [ (x2*(e8 + e2*x1 + e5*x2))/(x1*(e7 + e1*x1 + e4*x2)), (e7*e8 + e1*e2*x1^2 + e4*e5*x2^2 + 2*e2*e7*x1 + e4*e8*x2 + e5*e7*x2 + 2*e2*e4*x1*x2)/(x1*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2)), ((e8 + e2*x1 + e5*x2)*(e7*e8 + e1*e2*x1^2 + e4*e5*x2^2 + e1*e8*x1 + e2*e7*x1 + 2*e4*e8*x2 + 2*e2*e4*x1*x2))/(x1*(e7 + e1*x1 + e4*x2)*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2)), (e7*e8*e9 + e1*e8*e9*x1 + 2*e2*e7*e9*x1 + 2*e4*e8*e9*x2 + e5*e7*e9*x2 + e1*e2*e3*x1^3 + 2*e1*e2*e9*x1^2 + e2*e3*e7*x1^2 + e4*e5*e6*x2^3 + 2*e4*e5*e9*x2^2 + e4*e6*e8*x2^2 + e1*e6*e8*x1*x2 + 4*e2*e4*e9*x1*x2 + e3*e5*e7*x1*x2 + e1*e2*e6*x1^2*x2 + 2*e2*e3*e4*x1^2*x2 + 2*e2*e4*e6*x1*x2^2 + e3*e4*e5*x1*x2^2)/(x1*(e7 + e1*x1 + e4*x2)*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2))
    -x2/x1,                                                              -(e7 + e1*x1 + e4*x2)^2/(x1*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2)),                                         -(e7*e8 + e1*e2*x1^2 + e4*e5*x2^2 + 2*e1*e8*x1 + 2*e4*e8*x2 + e1*e5*x1*x2 + e2*e4*x1*x2)/(x1*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2)),                                                                                                                                                                                                                                             -(e7*e9 + e1*e3*x1^2 + e4*e6*x2^2 + 2*e1*e9*x1 + 2*e4*e9*x2 + e1*e6*x1*x2 + e3*e4*x1*x2)/(x1*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2))
    -(e8 + e2*x1 + e5*x2)/(e7 + e1*x1 + e4*x2),                                                    -(e4*e8 - e5*e7 - e1*e5*x1 + e2*e4*x1)/(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2),                                                  -((e8 + e2*x1 + e5*x2)*(e4*e8 - e5*e7 - e1*e5*x1 + e2*e4*x1))/((e7 + e1*x1 + e4*x2)*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2)),                                                                                                                                   -(e4*e8*e9 - e5*e7*e9 - 2*e1*e5*e9*x1 + e1*e6*e8*x1 + 2*e2*e4*e9*x1 - e2*e6*e7*x1 + e4*e6*e8*x2 - e5*e6*e7*x2 - e1*e3*e5*x1^2 + e2*e3*e4*x1^2 - e1*e5*e6*x1*x2 + e2*e4*e6*x1*x2)/((e7 + e1*x1 + e4*x2)*(e1*e8*x1 - e2*e7*x1 + e4*e8*x2 - e5*e7*x2))
    1,                                                                                                                                     0,                                                                                                                                                                                 0,                                                                                                                                                                                                                                                                                                                                                                                     0
    0,                                                                                                                                     1,                                                                                                                                                                                 0,                                                                                                                                                                                                                                                                                                                                                                                     0
    0,                                                                                                                                     0,                                                                                                                                                                                 1,                                                                                                                                                                                                                                                                                                                                                                                     0];
end